/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_CPU_OPERATOR_BATCHNORMIMPL_H__
#define __AIDGE_CPU_OPERATOR_BATCHNORMIMPL_H__

#include <array>
#include <memory>
#include <tuple>
#include <vector>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/BatchNorm.hpp"
#include "aidge/utils/Registrar.hpp"
#include "aidge/utils/Types.h"

namespace Aidge {
// class BatchNorm_Op;

// compute kernel registry for forward and backward
class BatchNormImpl2DForward_cpu
    : public Registrable<BatchNormImpl2DForward_cpu,
                         std::tuple<DataType, DataType, DataType>,
                         void(const BatchNorm_Op<2>::Parameters &,
                              const std::array<DimSize_t, 4> &,
                              const void *,
                              const void *,
                              const void *,
                              void *,
                              void *,
                              void *,
                              const bool)> {};
class BatchNormImpl2DBackward_cpu
    : public Registrable<BatchNormImpl2DBackward_cpu,
                         std::tuple<DataType, DataType, DataType>,
                         void(const BatchNorm_Op<2>::Parameters &,
                              const std::array<DimSize_t, 4> &,
                              const void *,
                              const void *,
                              const void *,
                              void *,
                              void *,
                              void *)> {};

class BatchNormImpl2D_cpu : public OperatorImpl {
   private:
    const BatchNorm_Op<2> &mOp;
    std::array<NbElts_t, 5> mNbConsumedData;
    std::array<NbElts_t, 1> mNbProducedData;

   public:
    BatchNormImpl2D_cpu(const BatchNorm_Op<2> &op) : mOp(op), mNbConsumedData({0, 0, 0, 0, 0}), mNbProducedData({0}) {}

    static std::unique_ptr<BatchNormImpl2D_cpu> create(const BatchNorm_Op<2> &op) {
        return std::make_unique<BatchNormImpl2D_cpu>(op);
    }

   public:
    NbElts_t getNbRequiredData(const IOIndex_t inputIdx) const override final;
    NbElts_t getNbRequiredProtected(const IOIndex_t inputIdx) const override final;
    NbElts_t getRequiredMemory(__attribute__((unused)) const IOIndex_t outputIdx, const std::vector<DimSize_t> &inputsSize) const override final;
    NbElts_t getNbConsumedData(const IOIndex_t inputIdx) const override final;
    NbElts_t getNbProducedData(const IOIndex_t outputIdx) const override final;

    void forward();

    void backward();
};

namespace {
// add cpu backend to BatchNorm_Op<2> implementation registry
static Registrar<BatchNorm_Op<2>> registrarBatchNormImpl2D_cpu("cpu", Aidge::BatchNormImpl2D_cpu::create);
}  // namespace
}  // namespace Aidge

#endif /* __AIDGE_CPU_OPERATOR_BATCHNORMIMPL_H__ */