/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef __AIDGE_CPU_OPERATOR_SOFTMAXIMPL_FORWARD_KERNEL_H__
#define __AIDGE_CPU_OPERATOR_SOFTMAXIMPL_FORWARD_KERNEL_H__

#include "aidge/utils/Registrar.hpp"
#include <cstddef>
#include <cmath>
#include "aidge/data/Data.hpp"
#include "aidge/utils/Types.h"

#include "aidge/operator/SoftmaxImpl.hpp"

namespace Aidge {
template <class I, class O>
void SoftmaxImpl_cpu_forward_kernel(const DimSize_t batchSize,
                                        const DimSize_t channelSize,
                                        const DimSize_t featureSize,
                                        const void* input_,
                                        void* output_) {

    const I* input = static_cast<const I*>(input_);
    O* output = static_cast<O*>(output_);

    for (std::size_t batch = 0; batch < batchSize; ++batch) {
        for (std::size_t feature = 0; feature < featureSize; ++feature) {
            std::size_t ioIndex = batch*channelSize*featureSize + feature;

            I sum(0.0);
            for (std::size_t ch = 0; ch < channelSize; ++ch) {
                output[ioIndex] = std::exp(input[ioIndex]);
                sum += output[ioIndex];
                ioIndex+=featureSize;
            }

            ioIndex = batch*channelSize*featureSize + feature;
            for (std::size_t ch = 0; ch < channelSize; ++ch) {
                output[ioIndex] /= sum;
                ioIndex += featureSize;
            }
        }
    }
}

namespace {
static Registrar<SoftmaxImplForward_cpu> registrarSoftmaxImplForward_cpu_Float32(
        {DataType::Float32, DataType::Float32}, Aidge::SoftmaxImpl_cpu_forward_kernel<float, float>);
static Registrar<SoftmaxImplForward_cpu> registrarSoftmaxImplForward_cpu_Int32(
        {DataType::Int32, DataType::Int32}, Aidge::SoftmaxImpl_cpu_forward_kernel<int, int>);
static Registrar<SoftmaxImplForward_cpu> registrarSoftmaxImplForward_cpu_Float64(
        {DataType::Float64, DataType::Float64}, Aidge::SoftmaxImpl_cpu_forward_kernel<double, double>);
}  // namespace
}  // namespace Aidge

#endif /* __AIDGE_CPU_OPERATOR_SOFTMAXIMPL_FORWARD_KERNEL_H__ */
